/**
 * Actualiza el elemento de la lista en caso de que exista en dicha lista, o lo inserta en caso de que no exista
 * @param element objeto que queremos actualizar o insertar en la lista
 * @param list lista de elementos
 */
export function upsertElement(element, list) {
  let find: boolean = false;
  if (!list) list = [];
  list.forEach((item, index) => {
    if (item.id == element.id) {
      list[index] = element;
      find = true;
    }
  });
  if (!find) {
    list.push(element);
  }
  return list;
}

/**
 * Elimina el elemento de la lista
 * @param element elemento a eliminar de la lista
 * @param list lista de elementos
 */
export function deleteElement(element, list) {
  return list.filter(item => item.id != element.id);
}

/** Permite copiar las propiedades de un objeto exceptuando las propiedades indicadas en keys.
 * Util para obtener los valores del filtro exceptuando las propiedades pagina y resultados.
 * La copia se realiza mediante Object.assign por lo que hay que tenerlo en cuenta por temas de propiedades anidadas
 * @param obj objeto que queremos copiar
 * @param keys array de propiedades que no queremos copiar
 */
export function reject(obj, keys) {
  return Object.keys(obj)
    .filter(k => !keys.includes(k))
    .map(k => Object.assign({}, { [k]: obj[k] }))
    .reduce((res, o) => Object.assign(res, o), {});
}

/** Permite seleccionar las propiedades de un objeto indicadas en keys, excluyendo el resto.
 * La copia se realiza mediante Object.assign por lo que hay que tenerlo en cuenta por temas de propiedades anidadas
 * @param obj objeto que queremos copiar
 * @param keys array de propiedades que queremos copiar
 */
export function choose(obj, keys) {
  return Object.keys(obj)
    .filter(k => keys.includes(k))
    .map(k => Object.assign({}, { [k]: obj[k] }))
    .reduce((res, o) => Object.assign(res, o), {});
}

/**
 * Equivalente al Object.assign() pero realiza una copia profunda: tanto los atributos como los objetos anidados
 * Modifica el objeto target asignando los objetos pasados como parámetros
 * ejemplo: assign({a: "a", b: {id: "b"}, c: "c"},
 *                 {a: "a2", b: {id: "b2"}, d: "d"}) =>
 *                 {a: "a2", b: {id: "b2"}, c: "c", d: "d"}
 * 
 * @param target objeto destino
 * @param sources objetos cuyas propiedades se leen para mergear en el objeto destino
 * @see https://stackoverflow.com/questions/27936772/how-to-deep-merge-instead-of-shallow-merge
 */
export function assign(target, ...sources) {
  const isObject = obj => obj && typeof obj === 'object';

  if (!sources.length) return target;
  const source = sources.shift();

  if (isObject(target) && isObject(source)) {
    for (const key in source) {
      if (isObject(source[key])) {
        if (!target[key]) target[key] = {};
        assign(target[key], source[key]);
      } else {
        target[key] = source[key];
      }
    }
  }

  return assign(target, ...sources);
}
/*
Versión immutable:

export function assign(...objects) {
  const isObject = obj => obj && typeof obj === 'object';

  return objects.reduce((prev, obj) => {
    Object.keys(obj).forEach(key => {
      const pVal = prev[key];
      const oVal = obj[key];
      
      if (Array.isArray(pVal) && Array.isArray(oVal)) {
        prev[key] = pVal.concat(...oVal);
      }
      else if (isObject(pVal) && isObject(oVal)) {
        prev[key] = assign(pVal, oVal);
      }
      else {
        prev[key] = oVal;
      }
    });
    
    return prev;
  }, {});
}*/